/* 
 * ShapeAdapter.java 1.1 2002-06-13
 *
 *  2002 Jouni Smed
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of the University of Turku nor the names of its
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package fi.utu.cs.appearance;

import java.awt.Shape;
import java.awt.Stroke;
import java.awt.Paint;
import java.awt.Color;

/**
 *  An abstract adapter class for defining shapable objects that can 
 *  be displayed in the <code>Graphics2D</code> context. The methods 
 *  in this class either return <code>false</code> or <code>null</code>. 
 *  This class exists as convenience for creating <code>Shapable</code> 
 *  objects. Extend this class to create a <code>Shapable</code> object 
 *  and override the methods of interest. 
 *
 *  @see Shapable
 *	@since 1.1
 *  @version 1.1, 020613
 *  @author  Jouni Smed
 */

public abstract class ShapeAdapter implements Shapable
  {  
  /**
   *  Returns a <code>Shape</code> object that defines the appearance
   *  of the object.
   */
  public Shape shape()
  	{
  	return null;
  	}

  /** 
   *  Returns a boolean value indicating whether the outline of the
   *  shape is drawn or not.
   *  @see #stroke()
   *  @see #color()
   */ 
  public boolean isDrawn()
  	{
  	return false;
  	}
  
  /**
   *  Returns a <code>Stroke</code> object that is used for drawing
   *  the outline. 
   *  @see #isDrawn()
   */
  public Stroke stroke()
  	{
  	return null;
  	}

  /**
   *  Returns a <code>Color</code> object that is used for drawing
   *  the outline. 
   *  @see #isDrawn()
   */
  public Color color()
  	{
  	return null;
  	}

  /**
   *  Returns a boolean value indicating whether the shape is filled
   *  or not.
   *  @see #paint()
   */
  public boolean isFilled()
  	{
  	return false;
  	}

  /**
   *  Returns a <code>Paint</code> object that is used for filling
   *  the shape. 
   *  @see #isFilled()
   */
  public Paint paint()
  	{
  	return null;
  	}
  }
