/* 
 * AIsHockeyServer.java 1.0 2002-11-25
 *
 *  2002 Jouni Smed
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of the University of Turku nor the names of its
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fi.utu.cs.hockey;

import fi.utu.cs.hockey.Game;
import fi.utu.cs.hockey.net.Server;
import fi.utu.cs.hockey.rink.FaceOff;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;


/**
 *  A hockey game server. 
 *
 *  @version 1.0, 021016
 *  @author  Jouni Smed
 */
public class AIsHockeyServer extends JFrame
  {
  private static final String[] 
           keywords = { "SERVER_PORT", "INIT_PORT",
                        "GROUP_ADDRESS", "GROUP_PORT",
                        "TIME", "UPDATE_FREQUENCY", "ZOOM", 
                        "STATUS_WIDTH", "FACEOFF_DELAY", 
                        "CONSOLE_ONLY", "UNSYNCHRONIZED",
                        "SILENT", "NO_LOG", "NO_STARTUP_MESSAGES",
                        "FULL_SCREEN", "HORIZONTAL_VIEW",
                        "FONT_NAME", "FONT_SIZE", 
                        "REPORT_TOURNAMENT", "TEXT_COLOR_HOME",
                        "TEXT_COLOR_VISITOR", "TEXT_COLOR_REFEREE"};
                        
  private static final int TAG_UNKNOWN = -1;
  private static final int TAG_SERVER_PORT = 0;
  private static final int TAG_INIT_PORT = 1;
  private static final int TAG_GROUP_ADDRESS = 2;
  private static final int TAG_GROUP_PORT = 3;
  private static final int TAG_TIME = 4;
  private static final int TAG_UPDATE_FREQUENCY = 5;
  private static final int TAG_ZOOM = 6;
  private static final int TAG_STATUS_WIDTH = 7;
  private static final int TAG_FACEOFF_DELAY = 8;
  private static final int TAG_CONSOLE_ONLY = 9;
  private static final int TAG_UNSYNCHRONIZED = 10;
  private static final int TAG_SILENT = 11;
  private static final int TAG_NO_LOG = 12;
  private static final int TAG_NO_STARTUP_MESSAGES = 13;
  private static final int TAG_FULL_SCREEN = 14;
  private static final int TAG_HORIZONTAL_VIEW = 15;
  private static final int TAG_FONT_NAME = 16;
  private static final int TAG_FONT_SIZE = 17;
  private static final int TAG_REPORT_TOURNAMENT = 18;
  private static final int TAG_TEXT_COLOR_HOME = 19;
  private static final int TAG_TEXT_COLOR_VISITOR = 20;
  private static final int TAG_TEXT_COLOR_REFEREE = 21;
  

  private int serverPort = 0;
  private int initPort = 0;
  private String groupAddress = "";
  private int groupPort = 0;
  private int time = 0;
  private int updateFrequency = 50;
  private double zoom = 16.0;
  private int statusWidth = 300;
  private int faceoffDelay = 2000;
  private boolean consoleOnly = false;
  private boolean unsynchronized = false;
  private boolean silent = false;
  private boolean noLog = false;
  private boolean noStartupMessages = false;
  private boolean fullScreen = false;
  private int fullScreenWidth;
  private int fullScreenHeight;
  private int fullScreenDepth;
  private boolean horizontalView = false;
  private String fontName = null;
  private int fontSize = 0;
  private boolean reportTournament = false;
  private Color textColorHome = Color.blue.brighter();
  private Color textColorVisitor = Color.red.brighter();
  private Color textColorReferee = Color.green.brighter();

  GraphicsDevice gfxDevice = null;
  DisplayMode oldDisplayMode = null;

  JPanel sidePanel;

  JPanel statusPanel;
  JLabel gameStatusTitle;
  JLabel gameStatusTime;
  JLabel gameStatusScore;
  JLabel gameStatusRoster;
  JLabel gameStatusReferee;

  JPanel faceoffPanel;  
  JButton[] faceoffButton = new JButton[9];
  int[] faceoffMnemonic =
    { KeyEvent.VK_V, KeyEvent.VK_I, 
      KeyEvent.VK_S, KeyEvent.VK_T,
      KeyEvent.VK_C,
      KeyEvent.VK_M, KeyEvent.VK_E,
      KeyEvent.VK_H, KeyEvent.VK_O };
  String[] faceoffText =
    { "Visitor endzone left", "Visitor endzone right",
      "Visitor neutral zone left", "Visitor neutral zone right",
      "Center circle",
      "Home neutral zone left", "Home neutral zone right",
      "Home endzone left", "Home endzone right" };
  int[] faceoffSpot =      
    { FaceOff.VISITOR_ENDZONE_LEFT, FaceOff.VISITOR_ENDZONE_RIGHT,
      FaceOff.VISITOR_NEUTRAL_ZONE_LEFT, FaceOff.VISITOR_NEUTRAL_ZONE_RIGHT,
      FaceOff.CENTER_SPOT,
      FaceOff.HOME_NEUTRAL_ZONE_LEFT, FaceOff.HOME_NEUTRAL_ZONE_RIGHT,
      FaceOff.HOME_ENDZONE_LEFT, FaceOff.HOME_ENDZONE_RIGHT };

  Server server;
  Game game;

  /** 
   *  Constructs a hockey game based on the initialization data 
   *  from the stream <code>in</code>.
   */
  protected AIsHockeyServer(InputStream in)
    {
    BufferedReader reader = new BufferedReader(new InputStreamReader(in));
    String row;   
    try
       {
       row = reader.readLine();
       while (row != null)
          {
          int index = TAG_UNKNOWN;
          for (int i = 0; i < keywords.length; i++)
            {
            if (row.startsWith(keywords[i]))
              {
              index = i;
              break;
              }
            }
          if (index != TAG_UNKNOWN)
            {
            String argument = row.substring(keywords[index].length());
            argument = argument.trim();
            switch (index) 
              {
              case TAG_SERVER_PORT:
                try
                  {
                  serverPort = Integer.parseInt(argument);
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Not a number: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_INIT_PORT:
                try
                  {
                  initPort = Integer.parseInt(argument);
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Not a number: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_GROUP_ADDRESS:
                groupAddress = argument;
                break;
              case TAG_GROUP_PORT:
                try
                  {
                  groupPort = Integer.parseInt(argument);
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Not a number: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_TIME:
                try
                  {
                  time = Integer.parseInt(argument);
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Not a number: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_UPDATE_FREQUENCY:
                try
                  {
                  updateFrequency = Integer.parseInt(argument);
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Not a number: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_ZOOM:
                try
                  {
                  zoom = Double.parseDouble(argument);
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Not a number: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_STATUS_WIDTH:
                try
                  {
                  statusWidth = Integer.parseInt(argument);
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Not a number: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_FACEOFF_DELAY:
                try
                  {
                  faceoffDelay = Integer.parseInt(argument);
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Not a number: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_CONSOLE_ONLY:
                consoleOnly = true;
                break;
              case TAG_UNSYNCHRONIZED:
                unsynchronized = true;
                break;
              case TAG_SILENT:
                silent = true;
                break;
              case TAG_NO_LOG:
                noLog = true;
                break;
              case TAG_NO_STARTUP_MESSAGES:
                noStartupMessages = true;
                break;
              case TAG_FULL_SCREEN:
                fullScreen = true;
                StringTokenizer ast = new StringTokenizer(argument);
                try
                  {
                  fullScreenWidth = Integer.parseInt(ast.nextToken());
                  fullScreenHeight = Integer.parseInt(ast.nextToken());
                  fullScreenDepth = Integer.parseInt(ast.nextToken());
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Contains not a number: " + argument);
                  System.exit(1);
                  }               
                catch (NoSuchElementException f)
                  {
                  System.err.println("Not enough parameters: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_HORIZONTAL_VIEW:
                horizontalView = true;
                break;
              case TAG_FONT_NAME:
                fontName = argument;
                break;               
              case TAG_FONT_SIZE:
                try
                  {
                  fontSize = Integer.parseInt(argument);
                  }
                catch (NumberFormatException e)
                  {
                  System.err.println("Not a number: " + argument);
                  System.exit(1);
                  }
                break;
              case TAG_REPORT_TOURNAMENT:
                reportTournament = true;
                break;
              case TAG_TEXT_COLOR_HOME:
								try
									{
									textColorHome = Color.decode(argument);
									}
								catch (NumberFormatException e)
									{
									System.err.println("Not a color: " + argument);
									System.exit(1);
									}
								break;
              case TAG_TEXT_COLOR_VISITOR:
								try
									{
									textColorVisitor = Color.decode(argument);
									}
								catch (NumberFormatException e)
									{
									System.err.println("Not a color: " + argument);
									System.exit(1);
									}
								break;
              case TAG_TEXT_COLOR_REFEREE:
								try
									{
									textColorReferee = Color.decode(argument);
									}
								catch (NumberFormatException e)
									{
									System.err.println("Not a color: " + argument);
									System.exit(1);
									}
								break;
              }           
            }
          row = reader.readLine();
          }
       reader.close();
       }
    catch (IOException e)
       {
       System.err.println("Configuration parse failed.");
       System.exit(1);   
       }
                
    if (!noStartupMessages)
      {
      System.out.println("Starting server");
      System.out.println("    Port: " + serverPort);
      System.out.println("    Init: " + initPort);
      System.out.println("  Group:  " + groupAddress);
      System.out.println("    Port: " + groupPort);
      System.out.println();
      System.out.println("Update frequency: " + updateFrequency + " ms");
      if (!consoleOnly)
        {
        System.out.println("Zoom: " + zoom);
        if (horizontalView) System.out.println("Horizontal view.");
        else System.out.println("Vertical view.");
        if (fullScreen)
          {
          System.out.println("Full screen:");
          System.out.println("  Width: " + fullScreenWidth);
          System.out.println("  Height: " + fullScreenHeight);
          System.out.println("  Depth: " + fullScreenDepth);
          System.out.println();
          if (fontName != null) System.out.println("Font name: " + fontName);
          if (fontSize > 0) System.out.println("Font size: " + fontSize);
          System.out.println("  Colors: " + textColorHome + ", " +
                             textColorVisitor + ", " + textColorReferee);
          }
        else
          {
          System.out.println("Status board width: " + statusWidth);
          }
        }
      else 
        {
        System.out.println("Console mode.");
        }
      System.out.println("Faceoff delay: " + faceoffDelay + " ms");
      System.out.println("Game duration: " + time + " s");
      if (unsynchronized)
        {
        System.out.println("Unsynchronized mode.");
        }
      if (silent)
        {
        System.out.println("Silent mode.");
        }
      if (noLog)
        {
        System.out.println("No log.");
        }
      }

    Game game = new Game((long)time * 1000L, updateFrequency, 
                          faceoffDelay);
    Server server = new Server(serverPort, initPort, 
                              groupAddress, groupPort, game);
      
    if (!consoleOnly)
      {
      game.prepareView(zoom, horizontalView);
      if (!silent) game.loadSounds();     
      }
    if (!consoleOnly && !fullScreen)
      {

      sidePanel = new JPanel();
      sidePanel.setLayout(new BorderLayout());

      statusPanel = new JPanel();
      statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.Y_AXIS));
      statusPanel.setPreferredSize(new Dimension(statusWidth, 10));

      gameStatusTitle = new JLabel("<html><h1>AIsHockey</h1></html>");
      gameStatusTitle.setHorizontalAlignment(SwingConstants.CENTER);
      gameStatusTime = new JLabel();
      gameStatusTime.setHorizontalAlignment(SwingConstants.CENTER);
      gameStatusScore = new JLabel();
      gameStatusScore.setHorizontalAlignment(SwingConstants.CENTER);
      gameStatusRoster = new JLabel();
      gameStatusRoster.setHorizontalAlignment(SwingConstants.CENTER);
      gameStatusReferee = new JLabel();
      gameStatusReferee.setHorizontalAlignment(SwingConstants.CENTER);

      statusPanel.add(gameStatusTitle);
      statusPanel.add(gameStatusTime);
      statusPanel.add(gameStatusScore);
      statusPanel.add(gameStatusRoster);
      statusPanel.add(gameStatusReferee);


      faceoffPanel = new JPanel();  
      faceoffPanel.setLayout(new BorderLayout());
      JPanel subPanel = null;
      for (int i = 0; i < 9; i++)
        {
        faceoffButton[i] = new JButton(faceoffText[i]);
        faceoffButton[i].setMnemonic(faceoffMnemonic[i]);
        faceoffButton[i].addActionListener(new Faceoffer(game, faceoffSpot[i]));
        switch (i)
          {
          case 0: 
            subPanel = new JPanel();
            subPanel.setLayout(new GridLayout(2, 2));
            faceoffPanel.add(subPanel, BorderLayout.NORTH);
          case 1:
          case 2:
          case 3:
            subPanel.add(faceoffButton[i]);
            break;
          case 4:
            faceoffPanel.add(faceoffButton[i], BorderLayout.CENTER);
            break;
          case 5:
            subPanel = new JPanel();
            subPanel.setLayout(new GridLayout(2, 2));
            faceoffPanel.add(subPanel, BorderLayout.SOUTH);
          case 6:
          case 7:
          case 8:
            subPanel.add(faceoffButton[i]);
            break;

          }
        }
      JPanel titlePanel = new JPanel();
      titlePanel.setLayout(new BorderLayout());
      JLabel titleLabel = new JLabel("<html><h2>Faceoff</h2></html>");
      titlePanel.add(titleLabel, BorderLayout.NORTH);
      titlePanel.add(faceoffPanel, BorderLayout.CENTER);

      sidePanel.add(statusPanel, BorderLayout.CENTER);
      sidePanel.add(titlePanel, BorderLayout.SOUTH);

      game.setGameStatus(gameStatusTime, gameStatusScore, 
                         gameStatusRoster, gameStatusReferee);
      getContentPane().add(game.getView(), BorderLayout.CENTER);    
      getContentPane().add(sidePanel, BorderLayout.EAST);    
      }

    if (!consoleOnly && fullScreen)
      {
      GraphicsEnvironment env = 
                    GraphicsEnvironment.getLocalGraphicsEnvironment();
      GraphicsDevice gfxDevice = env.getDefaultScreenDevice();
      
      if (!gfxDevice.isFullScreenSupported())
        {
        System.err.println("Error: No full screen support.");
        System.exit(1);
        }
/*      if (!gfxDevice.isDisplayChangeSupported())
        {
        System.err.println("Error: No display change support.");
        System.exit(1);
        }
*/
      oldDisplayMode = gfxDevice.getDisplayMode();
      DisplayMode displayMode = new DisplayMode(fullScreenWidth,
                              fullScreenHeight, fullScreenDepth, 0);

      GraphicsConfiguration gc = gfxDevice.getDefaultConfiguration();
      BufferCapabilities capabilities = gc.getBufferCapabilities();
      
      if (!capabilities.isPageFlipping())
        {
        System.err.println("Error: No page flipping support.");
        System.exit(1);
        }

      Frame mainFrame = new Frame(gc);
      mainFrame.setUndecorated(true);
      mainFrame.setIgnoreRepaint(true);
        
      gfxDevice.setFullScreenWindow(mainFrame);

      try
        {
        gfxDevice.setDisplayMode(displayMode);
        } 
      catch (Exception e) 
        {
        System.err.println("Error: Cannot change display mode. " + e);
        System.exit(1);
        } 
      
      mainFrame.createBufferStrategy(2);
      BufferStrategy bufferStrategy = mainFrame.getBufferStrategy();

      mainFrame.addKeyListener(new Keyer(game));
      
      game.setFullScreen(fullScreenWidth, fullScreenHeight, bufferStrategy,
                         fontName, fontSize, 
                         textColorHome, textColorVisitor, textColorReferee);
                         
      }

    if (unsynchronized) game.setUnsynchronized();
    if (noLog) game.setNoLog();
    if (reportTournament) game.setReportTournament();

    game.setServer(server);
    game.start();

    if (!consoleOnly & !fullScreen)
      {
      addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) { System.exit(0);} });
      setSize(800, 800);
      pack();
      setVisible(true);
      }
    if (consoleOnly)
      {
      new ConsoleFaceoffer(game, server).start();
      }
    }


  public void finalize() throws Throwable
    {
    if (fullScreen && gfxDevice != null)
      {
      gfxDevice.setDisplayMode(oldDisplayMode);
      gfxDevice.setFullScreenWindow(null);      
      }
    super.finalize();
    }


  private class Faceoffer implements ActionListener
    {
    private int spot;
    private Game game;
    
    public Faceoffer(Game g, int s)
      {
      game = g;
      spot = s;
      }
    
    public void actionPerformed(ActionEvent e)
      {
      game.whistleFaceoff(spot);
      }
    }


  class Keyer extends KeyAdapter
    {  
    private Game game;
    
    public Keyer(Game g)
      {
      game = g;
      }
    
    public void keyPressed(KeyEvent e)
      {
      switch (e.getKeyCode())
        {
        case KeyEvent.VK_1: 
          game.whistleFaceoff(FaceOff.HOME_ENDZONE_LEFT);
          break;
        case KeyEvent.VK_2: 
          game.whistleFaceoff(FaceOff.HOME_ENDZONE_RIGHT);
          break;
        case KeyEvent.VK_3: 
          game.whistleFaceoff(FaceOff.HOME_NEUTRAL_ZONE_LEFT);
          break;
        case KeyEvent.VK_4: 
          game.whistleFaceoff(FaceOff.HOME_NEUTRAL_ZONE_RIGHT);
          break;
        case KeyEvent.VK_5: 
        case KeyEvent.VK_SPACE:
          game.whistleFaceoff(FaceOff.CENTER_SPOT);
          break;
        case KeyEvent.VK_6: 
          game.whistleFaceoff(FaceOff.VISITOR_NEUTRAL_ZONE_LEFT);
          break;
        case KeyEvent.VK_7: 
          game.whistleFaceoff(FaceOff.VISITOR_NEUTRAL_ZONE_RIGHT);
          break;
        case KeyEvent.VK_8: 
          game.whistleFaceoff(FaceOff.VISITOR_ENDZONE_LEFT);
          break;
        case KeyEvent.VK_9: 
          game.whistleFaceoff(FaceOff.VISITOR_ENDZONE_RIGHT);
          break;
        case KeyEvent.VK_ESCAPE:
          if (fullScreen && gfxDevice != null)
            {
            gfxDevice.setDisplayMode(oldDisplayMode);
            gfxDevice.setFullScreenWindow(null);      
            }
          gfxDevice = null;
          System.exit(0);
          break;
        }
      }
    }


  private class ConsoleFaceoffer extends Thread
    {
    private Game game;
    private Server server;
    
    public ConsoleFaceoffer(Game g, Server s)
      {
      setDaemon(true);
      game = g;
      server = s;
      }
    
    public void run()
      {
      while (server.existsFreePosition()) 
        {
        yield();
        try 
          {
          sleep(50);
          }
        catch (InterruptedException e) { }
        }
      game.whistleFaceoff(FaceOff.CENTER_SPOT);
      }
    }
    
  }