/* 
 * Circle.java 1.0 2002-06-05
 *
 *  2002 Jouni Smed
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of the University of Turku nor the names of its
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
package fi.utu.cs.hockey.rink;

import fi.utu.cs.physics.*;
import fi.utu.cs.appearance.*;
import java.awt.Shape;
import java.awt.Paint;
import java.awt.Color;
import java.awt.Stroke;
import java.awt.BasicStroke;
import java.awt.geom.Ellipse2D;

/**
 *  A circle. 
 *
 *  @version 1.0, 020605
 *  @author  Jouni Smed
 */
public abstract class Circle extends BoundaryCurve implements Shapable
  {
  /** The stroke used in the circle. */
  private Stroke stroke;
  /** The color of the circle. */
  private Color color;

  /** 
   *  A circle with a center at the point (<code>px</code>, <code>py</code>)
   *  radius <code>r</code>, width <code>w</code> and color <code>c</code>.
   */
  protected Circle(double px, double py, double r, double w, Color c)
    {
    super(px, py, r);
    stroke = new BasicStroke((float)w);
    color = c;
    setEthereal();
    }
  
  // ***************************************************
  // Methods defined in the Shapable interface  
  // ***************************************************

  public Shape shape()
    {
    double x = getCenter().getX() - getRadius();
    double y = getCenter().getY() - getRadius();
    double w = 2 * getRadius();
    double h = 2 * getRadius();

		return new Ellipse2D.Double(x, y, w, h);   
    }
    
  public boolean isDrawn()
    {
    return true;
    }
      
  public Stroke stroke()
    {
    return stroke;
    }

  public Color color()
    {
    return color;
    }

  public boolean isFilled()
    {
    return false;
    }

  public Paint paint()
    {
    return color;
    }  
  }
