/* 
 * GoalPost.java 1.0 2002-06-06
 *
 *  2002 Jouni Smed
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of the University of Turku nor the names of its
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
package fi.utu.cs.hockey.rink;

import fi.utu.cs.physics.*;
import fi.utu.cs.appearance.*;
import java.awt.Shape;
import java.awt.Paint;
import java.awt.Color;
import java.awt.Stroke;
import java.awt.BasicStroke;
import java.awt.geom.Ellipse2D;

/**
 *  A goal post. 
 *
 *  @version 1.0, 020606
 *  @author  Jouni Smed
 */
public class GoalPost extends BoundaryCurve implements Shapable
  {

  /** 
   *  A goal post.
   *  @param half In which half of the rink the goal post is.
   *  @param side In which side of the rink the goal post is.
   */
  public GoalPost(boolean half, boolean side)
    {
    super(side ? (Rink.GOAL_WIDTH + Rink.GOAL_POST_WIDTH) / 2.0 : 
    						-(Rink.GOAL_WIDTH + Rink.GOAL_POST_WIDTH) / 2.0, 
    			half ? Rink.GOAL_LINE_OFFSET : -Rink.GOAL_LINE_OFFSET, 
    			Rink.GOAL_POST_WIDTH / 2.0);
    }

  
  // ***************************************************
  // Methods defined in the Shapable interface  
  // ***************************************************

  public Shape shape()
    {
    double x = getCenter().getX() - getRadius();
    double y = getCenter().getY() - getRadius();
    double w = 2 * getRadius();
    double h = 2 * getRadius();

		return new Ellipse2D.Double(x, y, w, h);
    }
    
  public boolean isDrawn()
    {
    return false;
    }
      
  public Stroke stroke()
    {
    return null;
    }

  public Color color()
    {
    return Rink.COLOR_RED;
    }

  public boolean isFilled()
    {
    return true;
    }

  public Paint paint()
    {
    return Rink.COLOR_RED;
    }
  
  public String toString()
    {
    return "Goal post: " + super.toString();
    }  
  }
