/* 
 * Friction.java 1.1 2002-05-06
 *
 *  2002 Jouni Smed
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of the University of Turku nor the names of its
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package fi.utu.cs.physics;

/**
 *  A pair of static and kinetic friction coefficients.
 *
 *	@since 1.1
 *  @version 1.1, 020506
 *  @author  Jouni Smed
 */

public class Friction extends Tuple2
  {
  /** A friction with a zero static and dynamic coefficients. */
  public static final Friction ZERO = new Friction(0.0, 0.0);

  /** 
   *  Creates a new friction object with static friction coefficient
   *  <code>s</code> and kinetic friction coefficient <code>k</code>.  
   */
  public Friction(double s, double k)
    {
    super(s, k);
    }

  /** Returns the static friction coefficient. */
  public double getStaticCoefficient()
    {
    return getFirstElement();
    }

  /** Returns the kinetic friction coefficient. */
  public double getKineticCoefficient()
    {
    return getSecondElement();
    }

  /** Sets the static friction coefficient to the value <code>s</code>. */
  public void setStaticCoefficient(double s)
    {
    setFirstElement(s);
    }

  /** Sets the kinetic friction coefficient to the value <code>k</code>. */
  public void setKineticCoefficient(double k)
    {
    setSecondElement(k);
    }

  /**
   *  Sets the friction coefficients.
   *
   *  @param s The static friction coefficient.
   *  @param k The kinetic friction coefficient.
   *  @see #set(Friction)
   **/
  public void set(double s, double k)
    {
    super.set(s, k);
    }
    
  /** 
   *  Sets the friction coeffients to the same values as 
   *  in <code>other</code>. 
   */
  public void set(Friction other)
    {
    super.set(other);
    }    
    
  }
